# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .event_base import EventBase

__all__ = [
    "Event",
    "TransformEvent",
    "RouteEvent",
    "SplitCollectionEvent",
    "SplitCollectionEventPrintPageOutput",
    "SplitCollectionEventPrintPageOutputItem",
    "SplitCollectionEventSemanticPageOutput",
    "SplitCollectionEventSemanticPageOutputItem",
    "SplitItemEvent",
    "SplitItemEventPrintPageOutput",
    "SplitItemEventSemanticPageOutput",
    "ErrorEvent",
    "JoinEvent",
    "JoinEventItem",
]


class TransformEvent(EventBase):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """
    Unique identifier of action type configuration that this event is associated
    with.
    """

    item_count: int = FieldInfo(alias="itemCount")
    """The number of items that were transformed.

    Used for batch transformations to indicate how many items were transformed.
    """

    item_offset: int = FieldInfo(alias="itemOffset")
    """The offset of the first item that was transformed.

    Used for batch transformations to indicate which item in the batch this event
    corresponds to.
    """

    transformed_content: object = FieldInfo(alias="transformedContent")
    """The transformed content of the input.

    The structure of this object is defined by the action type configuration.
    """

    event_type: Optional[Literal["transform"]] = FieldInfo(alias="eventType", default=None)

    invalid_properties: Optional[List[str]] = FieldInfo(alias="invalidProperties", default=None)
    """List of properties that were invalid in the input."""


class RouteEvent(EventBase):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """
    Unique identifier of action type configuration that this event is associated
    with.
    """

    choice: str
    """The choice made by the router action."""

    event_type: Optional[Literal["route"]] = FieldInfo(alias="eventType", default=None)

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)
    """The presigned S3 URL of the file that was routed."""


class SplitCollectionEventPrintPageOutputItem(BaseModel):
    item_offset: Optional[int] = FieldInfo(alias="itemOffset", default=None)

    item_reference_id: Optional[str] = FieldInfo(alias="itemReferenceID", default=None)

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)
    """The presigned S3 URL of the file that was split."""


class SplitCollectionEventPrintPageOutput(BaseModel):
    item_count: Optional[int] = FieldInfo(alias="itemCount", default=None)

    items: Optional[List[SplitCollectionEventPrintPageOutputItem]] = None


class SplitCollectionEventSemanticPageOutputItem(BaseModel):
    item_class: Optional[str] = FieldInfo(alias="itemClass", default=None)

    item_class_count: Optional[int] = FieldInfo(alias="itemClassCount", default=None)

    item_class_offset: Optional[int] = FieldInfo(alias="itemClassOffset", default=None)

    item_offset: Optional[int] = FieldInfo(alias="itemOffset", default=None)

    item_reference_id: Optional[str] = FieldInfo(alias="itemReferenceID", default=None)

    page_end: Optional[int] = FieldInfo(alias="pageEnd", default=None)

    page_start: Optional[int] = FieldInfo(alias="pageStart", default=None)

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)


class SplitCollectionEventSemanticPageOutput(BaseModel):
    item_count: Optional[int] = FieldInfo(alias="itemCount", default=None)

    items: Optional[List[SplitCollectionEventSemanticPageOutputItem]] = None

    page_count: Optional[int] = FieldInfo(alias="pageCount", default=None)


class SplitCollectionEvent(EventBase):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """
    Unique identifier of action type configuration that this event is associated
    with.
    """

    output_type: Literal["print_page", "semantic_page"] = FieldInfo(alias="outputType")

    print_page_output: SplitCollectionEventPrintPageOutput = FieldInfo(alias="printPageOutput")

    semantic_page_output: SplitCollectionEventSemanticPageOutput = FieldInfo(alias="semanticPageOutput")

    event_type: Optional[Literal["split_collection"]] = FieldInfo(alias="eventType", default=None)


class SplitItemEventPrintPageOutput(BaseModel):
    collection_reference_id: Optional[str] = FieldInfo(alias="collectionReferenceID", default=None)

    item_count: Optional[int] = FieldInfo(alias="itemCount", default=None)

    item_offset: Optional[int] = FieldInfo(alias="itemOffset", default=None)

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)
    """The presigned S3 URL of the file that was split."""


class SplitItemEventSemanticPageOutput(BaseModel):
    collection_reference_id: Optional[str] = FieldInfo(alias="collectionReferenceID", default=None)

    item_class: Optional[str] = FieldInfo(alias="itemClass", default=None)

    item_class_count: Optional[int] = FieldInfo(alias="itemClassCount", default=None)

    item_class_offset: Optional[int] = FieldInfo(alias="itemClassOffset", default=None)

    item_count: Optional[int] = FieldInfo(alias="itemCount", default=None)

    item_offset: Optional[int] = FieldInfo(alias="itemOffset", default=None)

    page_count: Optional[int] = FieldInfo(alias="pageCount", default=None)

    page_end: Optional[int] = FieldInfo(alias="pageEnd", default=None)

    page_start: Optional[int] = FieldInfo(alias="pageStart", default=None)

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)


class SplitItemEvent(EventBase):
    output_type: Literal["print_page", "semantic_page"] = FieldInfo(alias="outputType")

    print_page_output: Optional[SplitItemEventPrintPageOutput] = FieldInfo(alias="printPageOutput", default=None)

    semantic_page_output: Optional[SplitItemEventSemanticPageOutput] = FieldInfo(
        alias="semanticPageOutput", default=None
    )


class ErrorEvent(EventBase):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """
    Unique identifier of action type configuration that this event is associated
    with.
    """

    message: str
    """Error message."""

    event_type: Optional[Literal["error"]] = FieldInfo(alias="eventType", default=None)


class JoinEventItem(BaseModel):
    item_count: int = FieldInfo(alias="itemCount")
    """The number of items that were transformed."""

    item_offset: int = FieldInfo(alias="itemOffset")
    """The offset of the first item that was transformed.

    Used for batch transformations to indicate which item in the batch this event
    corresponds to.
    """

    item_reference_id: str = FieldInfo(alias="itemReferenceID")
    """The unique ID you use internally to refer to this data point."""

    s3_url: Optional[str] = FieldInfo(alias="s3URL", default=None)
    """The presigned S3 URL of the file that was joined."""


class JoinEvent(EventBase):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """
    Unique identifier of action type configuration that this event is associated
    with.
    """

    invalid_properties: List[str] = FieldInfo(alias="invalidProperties")
    """List of properties that were invalid in the input."""

    items: List[JoinEventItem]
    """The items that were joined."""

    join_type: Literal["standard"] = FieldInfo(alias="joinType")
    """The type of join that was performed."""

    transformed_content: object = FieldInfo(alias="transformedContent")
    """The transformed content of the input.

    The structure of this object is defined by the action type configuration.
    """

    event_type: Optional[Literal["join"]] = FieldInfo(alias="eventType", default=None)


Event: TypeAlias = Union[TransformEvent, RouteEvent, SplitCollectionEvent, SplitItemEvent, ErrorEvent, JoinEvent]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .subscription import Subscription

__all__ = ["SubscriptionListResponse"]

SubscriptionListResponse: TypeAlias = List[Subscription]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ActionCorrectRouteParams", "RouteAction"]


class ActionCorrectRouteParams(TypedDict, total=False):
    route_actions: Annotated[Iterable[RouteAction], PropertyInfo(alias="routeActions")]
    """
    An array of objects containing all the route actions you want to submit feedback
    for.
    """


class RouteAction(TypedDict, total=False):
    action_id: Annotated[str, PropertyInfo(alias="actionID")]
    """The action ID of the relevant route action."""

    corrected_choice: Annotated[str, PropertyInfo(alias="correctedChoice")]
    """The desired route that should have been chosen."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["UpdateTransformationResponse", "Result"]


class Result(BaseModel):
    error: Optional[str] = None
    """error message"""

    success: Optional[bool] = None

    transformation_id: Optional[str] = FieldInfo(alias="transformationID", default=None)
    """The unique ID you use internally to refer to a transform."""


class UpdateTransformationResponse(BaseModel):
    results: Optional[List[Result]] = None
    """An array of objects containing all the transformations you want to patch."""

    success_count: Optional[int] = FieldInfo(alias="successCount", default=None)

    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransformationCreateResponse", "Transformation"]


class Transformation(BaseModel):
    reference_id: Optional[str] = FieldInfo(alias="referenceID", default=None)
    """The unique ID you use internally to refer to this data point."""

    task_create_status: Optional[str] = None
    """Whether the task is a newly created task or a duplicate task."""

    task_id: Optional[str] = FieldInfo(alias="taskID", default=None)
    """
    Task ID generated by bem to track the status of this transformation request
    using the tasks API.
    """


class TransformationCreateResponse(BaseModel):
    transformations: Optional[List[Transformation]] = None
    """
    An array of transformations that have been created or de-duplicated by
    reference.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .action_type_config import ActionTypeConfig

__all__ = ["ActionTypeConfigListResponse"]


class ActionTypeConfigListResponse(BaseModel):
    action_type_configs: Optional[List[ActionTypeConfig]] = FieldInfo(alias="actionTypeConfigs", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import (
    ActionTypeConfig,
    ActionTypeConfigListResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActionTypeConfigs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_1(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_1(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_1(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_2(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_2(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_2(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_3(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_3(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_3(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_4(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_4(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_4(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_overload_5(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_overload_5(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_overload_5(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.retrieve(
            "actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.retrieve(
            "actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.retrieve(
            "actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_1(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            complex_tabular_transform_enabled=True,
            independent_document_processing_enabled=False,
            name="name",
            next_action_type_config_id="nextActionTypeConfigID",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight Load Schema",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_1(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_1(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_1(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_2(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_2(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            description="description",
            name="name",
            routes=[
                {
                    "name": "name",
                    "action_type_config_id": "actionTypeConfigID",
                    "description": "description",
                    "origin": {"email": {"patterns": ["string"]}},
                    "regex": {"patterns": ["string"]},
                }
            ],
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_2(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_2(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_2(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_3(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_3(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            name="name",
            print_page_split_config={"next_action_type_config_id": "nextActionTypeConfigID"},
            semantic_page_split_config={
                "item_classes": [
                    {
                        "name": "name",
                        "description": "description",
                        "next_action_type_config_id": "nextActionTypeConfigID",
                    }
                ]
            },
            split_type="splitType",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_3(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_3(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_3(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_4(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_4(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            join_type="standard",
            name="name",
            next_action_type_config_id="nextActionTypeConfigID",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight Load Schema",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_4(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_4(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_4(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_5(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_5(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            body="body",
            from_email="dev@stainless.com",
            from_name="fromName",
            name="name",
            subject="subject",
            to_email="dev@stainless.com",
            to_name="toName",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_5(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_5(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_5(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.list()
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.list(
            action_type="transform",
            action_type_config_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: BemSDK) -> None:
        action_type_config = client.action_type_configs.delete(
            "actionTypeConfigID",
        )
        assert action_type_config is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: BemSDK) -> None:
        response = client.action_type_configs.with_raw_response.delete(
            "actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = response.parse()
        assert action_type_config is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: BemSDK) -> None:
        with client.action_type_configs.with_streaming_response.delete(
            "actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = response.parse()
            assert action_type_config is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            client.action_type_configs.with_raw_response.delete(
                "",
            )


class TestAsyncActionTypeConfigs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_3(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_4(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.create(
            body={"action_type": "transform"},
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.create(
            body={"action_type": "transform"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_overload_5(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.create(
            body={"action_type": "transform"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.retrieve(
            "actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.retrieve(
            "actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.retrieve(
            "actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            complex_tabular_transform_enabled=True,
            independent_document_processing_enabled=False,
            name="name",
            next_action_type_config_id="nextActionTypeConfigID",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight Load Schema",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_2(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            description="description",
            name="name",
            routes=[
                {
                    "name": "name",
                    "action_type_config_id": "actionTypeConfigID",
                    "description": "description",
                    "origin": {"email": {"patterns": ["string"]}},
                    "regex": {"patterns": ["string"]},
                }
            ],
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_3(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_3(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            name="name",
            print_page_split_config={"next_action_type_config_id": "nextActionTypeConfigID"},
            semantic_page_split_config={
                "item_classes": [
                    {
                        "name": "name",
                        "description": "description",
                        "next_action_type_config_id": "nextActionTypeConfigID",
                    }
                ]
            },
            split_type="splitType",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_3(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_3(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_3(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_4(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_4(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            join_type="standard",
            name="name",
            next_action_type_config_id="nextActionTypeConfigID",
            output_schema={
                "value": {
                    "type": "object",
                    "required": ["tenders"],
                    "properties": {
                        "tenders": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "required": [
                                    "loadReference",
                                    "origin",
                                    "destination",
                                    "weightTons",
                                    "loadType",
                                    "desiredDeliveryDate",
                                    "bidSubmissionDeadline",
                                    "submitter",
                                ],
                                "properties": {
                                    "origin": {
                                        "type": "string",
                                        "description": "The starting point of the shipment.",
                                    },
                                    "loadType": {
                                        "type": "string",
                                        "description": "The type of goods being shipped.",
                                    },
                                    "submitter": {
                                        "type": "object",
                                        "required": ["name", "position", "contactInfo"],
                                        "properties": {
                                            "name": {
                                                "type": "string",
                                                "description": "Name of the person submitting the tender.",
                                            },
                                            "position": {
                                                "type": "string",
                                                "description": "Position of the submitter within their company.",
                                            },
                                            "contactInfo": {
                                                "type": "object",
                                                "required": ["email"],
                                                "properties": {
                                                    "email": {
                                                        "type": "string",
                                                        "format": "email",
                                                        "description": "Email address of the submitter.",
                                                    },
                                                    "phone": {
                                                        "type": "string",
                                                        "description": "Phone number of the submitter.",
                                                    },
                                                },
                                            },
                                        },
                                    },
                                    "weightTons": {
                                        "type": "number",
                                        "description": "The weight of the load in tons.",
                                    },
                                    "destination": {
                                        "type": "string",
                                        "description": "The endpoint of the shipment.",
                                    },
                                    "loadReference": {
                                        "type": "string",
                                        "description": "Unique identifier for the load tender.",
                                    },
                                    "desiredDeliveryDate": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The preferred date for the shipment to be delivered.",
                                    },
                                    "bidSubmissionDeadline": {
                                        "type": "string",
                                        "format": "date",
                                        "description": "The deadline for submitting bids.",
                                    },
                                },
                            },
                        }
                    },
                }
            },
            output_schema_name="Freight Load Schema",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_4(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_4(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_4(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_5(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_5(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.update(
            action_type_config_id="actionTypeConfigID",
            body="body",
            from_email="dev@stainless.com",
            from_name="fromName",
            name="name",
            subject="subject",
            to_email="dev@stainless.com",
            to_name="toName",
        )
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_5(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.update(
            action_type_config_id="actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_5(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.update(
            action_type_config_id="actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfig, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_5(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.update(
                action_type_config_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.list()
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.list(
            action_type="transform",
            action_type_config_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert_matches_type(ActionTypeConfigListResponse, action_type_config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncBemSDK) -> None:
        action_type_config = await async_client.action_type_configs.delete(
            "actionTypeConfigID",
        )
        assert action_type_config is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.action_type_configs.with_raw_response.delete(
            "actionTypeConfigID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action_type_config = await response.parse()
        assert action_type_config is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBemSDK) -> None:
        async with async_client.action_type_configs.with_streaming_response.delete(
            "actionTypeConfigID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action_type_config = await response.parse()
            assert action_type_config is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `action_type_config_id` but received ''"):
            await async_client.action_type_configs.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .any_type import AnyType
from .input_type import InputType

__all__ = [
    "TransformationListResponse",
    "Transformation",
    "TransformationTransformedContent",
    "TransformationTransformedContentOutput",
]


class TransformationTransformedContentOutput(BaseModel):
    output: Optional[List[Optional[AnyType]]] = None


TransformationTransformedContent: TypeAlias = Union[
    TransformationTransformedContentOutput, List[object], str, float, bool, object
]


class Transformation(BaseModel):
    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp indicating when the transformation was created."""

    input_type: InputType = FieldInfo(alias="inputType")
    """The input type of the content you're sending for transformation."""

    item_offset: int = FieldInfo(alias="itemOffset")
    """
    For CSV transformations, this offset represents starting row of the CSV for
    which you want to fetch transformations for. For PDF transformations, this
    offset represents the starting page of the PDF that the transformation pertains
    to. For pipelines that have `independentDocumentProcessingEnabled` set to
    `true`. Note that this is zero-indexed.
    """

    pipeline_id: str = FieldInfo(alias="pipelineID")
    """ID of pipeline that transformed the original input data."""

    reference_id: str = FieldInfo(alias="referenceID")
    """The unique ID you use internally to refer to this data point."""

    transformation_id: str = FieldInfo(alias="transformationID")
    """Unique ID generated by bem following Segment's KSUID conventions."""

    transformed_content: TransformationTransformedContent = FieldInfo(alias="transformedContent")
    """The transformed content as an object, in the schema defined by the pipeline."""

    invalid_properties: Optional[List[str]] = FieldInfo(alias="invalidProperties", default=None)
    """
    Highlights any fields missing from transforming the original input data that are
    marked as `required` in the respective output schema for the pipeline.
    """

    last_publish_error_at: Optional[str] = FieldInfo(alias="lastPublishErrorAt", default=None)
    """
    Last timestamp indicating when the transform was published via webhook and
    received a non-200 response. Set to `null` if on a subsequent retry if the
    webhook service receives a 200 response.
    """

    published_at: Optional[datetime] = FieldInfo(alias="publishedAt", default=None)
    """
    Timestamp indicating when the transform was published via webhook and received a
    successful 200 response. Value is `null` if the transformation hasn't been sent.
    """


class TransformationListResponse(BaseModel):
    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)
    """The total number of results available."""

    transformations: Optional[List[Transformation]] = None
    """An array of objects containing the transformations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["WebhookSecret"]


class WebhookSecret(BaseModel):
    secret: str
    """Webhook secret for your account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam
from .split_config_semantic_page_item_class_param import SplitConfigSemanticPageItemClassParam

__all__ = [
    "UpsertSplitConfigParam",
    "UpsertSplitConfigParamPrintPageSplitConfig",
    "UpsertSplitConfigParamSemanticPageSplitConfig",
]


class UpsertSplitConfigParamPrintPageSplitConfig(TypedDict, total=False):
    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """The unique ID of the action type configuration you want to use for this action."""


class UpsertSplitConfigParamSemanticPageSplitConfig(TypedDict, total=False):
    item_classes: Annotated[Iterable[SplitConfigSemanticPageItemClassParam], PropertyInfo(alias="itemClasses")]


class UpsertSplitConfigParam(ActionTypeConfigUpsertBaseParam, total=False):
    print_page_split_config: Annotated[
        UpsertSplitConfigParamPrintPageSplitConfig, PropertyInfo(alias="printPageSplitConfig")
    ]

    semantic_page_split_config: Annotated[
        UpsertSplitConfigParamSemanticPageSplitConfig, PropertyInfo(alias="semanticPageSplitConfig")
    ]

    split_type: Annotated[str, PropertyInfo(alias="splitType")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ActionTypeConfigBase"]


class ActionTypeConfigBase(BaseModel):
    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")
    """Unique identifier of action type config."""

    name: str
    """Name of action type config."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import (
    TransformationListResponse,
    TransformationCreateResponse,
    TransformationDeleteResponse,
    UpdateTransformationResponse,
    TransformationListErrorsResponse,
)
from bem._utils import parse_datetime
from tests.utils import assert_matches_type

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTransformations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: BemSDK) -> None:
        transformation = client.transformations.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        )
        assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: BemSDK) -> None:
        response = client.transformations.with_raw_response.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: BemSDK) -> None:
        with client.transformations.with_streaming_response.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = response.parse()
            assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: BemSDK) -> None:
        transformation = client.transformations.update()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: BemSDK) -> None:
        transformation = client.transformations.update(
            transformations=[
                {
                    "corrected_json": {},
                    "order_matching": True,
                    "transformation_id": "transformationID",
                }
            ],
        )
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: BemSDK) -> None:
        response = client.transformations.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: BemSDK) -> None:
        with client.transformations.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = response.parse()
            assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: BemSDK) -> None:
        transformation = client.transformations.list()
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: BemSDK) -> None:
        transformation = client.transformations.list(
            ending_before="endingBefore",
            item_offset=0,
            limit=1,
            pipeline_id="pipelineID",
            published=True,
            published_after=parse_datetime("2019-12-27T18:11:19.117Z"),
            published_before=parse_datetime("2019-12-27T18:11:19.117Z"),
            reference_ids=["string"],
            sort_order="asc",
            starting_after="startingAfter",
            transformation_ids=["string"],
        )
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: BemSDK) -> None:
        response = client.transformations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: BemSDK) -> None:
        with client.transformations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = response.parse()
            assert_matches_type(TransformationListResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: BemSDK) -> None:
        transformation = client.transformations.delete()
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: BemSDK) -> None:
        transformation = client.transformations.delete(
            pipeline_id="pipelineID",
            reference_ids=["string"],
            transformation_ids=["string"],
        )
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: BemSDK) -> None:
        response = client.transformations.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: BemSDK) -> None:
        with client.transformations.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = response.parse()
            assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_deprecated_update(self, client: BemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            transformation = client.transformations.deprecated_update()

        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_deprecated_update_with_all_params(self, client: BemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            transformation = client.transformations.deprecated_update(
                transformations=[
                    {
                        "corrected_json": {},
                        "order_matching": True,
                        "transformation_id": "transformationID",
                    }
                ],
            )

        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_deprecated_update(self, client: BemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.transformations.with_raw_response.deprecated_update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_deprecated_update(self, client: BemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            with client.transformations.with_streaming_response.deprecated_update() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                transformation = response.parse()
                assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list_errors(self, client: BemSDK) -> None:
        transformation = client.transformations.list_errors(
            reference_ids=["string"],
        )
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_errors_with_all_params(self, client: BemSDK) -> None:
        transformation = client.transformations.list_errors(
            reference_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            pipeline_id="pipelineID",
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_errors(self, client: BemSDK) -> None:
        response = client.transformations.with_raw_response.list_errors(
            reference_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = response.parse()
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_errors(self, client: BemSDK) -> None:
        with client.transformations.with_streaming_response.list_errors(
            reference_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = response.parse()
            assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTransformations:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        )
        assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.transformations.with_raw_response.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBemSDK) -> None:
        async with async_client.transformations.with_streaming_response.create(
            pipeline_id="pl_2c9AXIj48cUYJtCuv1gsQtHGDzK",
            transformations=[
                {
                    "input_content": "U3RhaW5sZXNzIHJvY2tz",
                    "input_type": "email",
                    "reference_id": "referenceID",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = await response.parse()
            assert_matches_type(TransformationCreateResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.update()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.update(
            transformations=[
                {
                    "corrected_json": {},
                    "order_matching": True,
                    "transformation_id": "transformationID",
                }
            ],
        )
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.transformations.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncBemSDK) -> None:
        async with async_client.transformations.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = await response.parse()
            assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.list()
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.list(
            ending_before="endingBefore",
            item_offset=0,
            limit=1,
            pipeline_id="pipelineID",
            published=True,
            published_after=parse_datetime("2019-12-27T18:11:19.117Z"),
            published_before=parse_datetime("2019-12-27T18:11:19.117Z"),
            reference_ids=["string"],
            sort_order="asc",
            starting_after="startingAfter",
            transformation_ids=["string"],
        )
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.transformations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(TransformationListResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBemSDK) -> None:
        async with async_client.transformations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = await response.parse()
            assert_matches_type(TransformationListResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.delete()
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.delete(
            pipeline_id="pipelineID",
            reference_ids=["string"],
            transformation_ids=["string"],
        )
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.transformations.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBemSDK) -> None:
        async with async_client.transformations.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = await response.parse()
            assert_matches_type(TransformationDeleteResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_deprecated_update(self, async_client: AsyncBemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            transformation = await async_client.transformations.deprecated_update()

        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_deprecated_update_with_all_params(self, async_client: AsyncBemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            transformation = await async_client.transformations.deprecated_update(
                transformations=[
                    {
                        "corrected_json": {},
                        "order_matching": True,
                        "transformation_id": "transformationID",
                    }
                ],
            )

        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_deprecated_update(self, async_client: AsyncBemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.transformations.with_raw_response.deprecated_update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_deprecated_update(self, async_client: AsyncBemSDK) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.transformations.with_streaming_response.deprecated_update() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                transformation = await response.parse()
                assert_matches_type(UpdateTransformationResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_errors(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.list_errors(
            reference_ids=["string"],
        )
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_errors_with_all_params(self, async_client: AsyncBemSDK) -> None:
        transformation = await async_client.transformations.list_errors(
            reference_ids=["string"],
            ending_before="endingBefore",
            limit=1,
            pipeline_id="pipelineID",
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_errors(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.transformations.with_raw_response.list_errors(
            reference_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transformation = await response.parse()
        assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_errors(self, async_client: AsyncBemSDK) -> None:
        async with async_client.transformations.with_streaming_response.list_errors(
            reference_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transformation = await response.parse()
            assert_matches_type(TransformationListErrorsResponse, transformation, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from .._utils import PropertyInfo
from .._models import BaseModel
from .route_list_item import RouteListItem
from .action_type_config_base import ActionTypeConfigBase
from .split_config_semantic_page_item_class import SplitConfigSemanticPageItemClass

__all__ = [
    "ActionTypeConfig",
    "TransformConfig",
    "RouteConfig",
    "SplitConfig",
    "SplitConfigPrintPageSplitConfig",
    "SplitConfigSemanticPageSplitConfig",
    "JoinConfig",
    "EmailConfig",
]


class TransformConfig(ActionTypeConfigBase):
    action_type: Literal["transform"] = FieldInfo(alias="actionType")

    complex_tabular_transform_enabled: bool = FieldInfo(alias="complexTabularTransformEnabled")
    """Whether complex tabular transforms are enabled on the pipeline.

    This enables the pipeline to parse CSVs with multiple tables in the same file,
    and to transpose CSVs that can't be parsed row-wise.
    """

    email_address: str = FieldInfo(alias="emailAddress")
    """Email address automatically created by bem.

    You can forward emails with or without attachments, to be transformed.
    """

    independent_document_processing_enabled: bool = FieldInfo(alias="independentDocumentProcessingEnabled")
    """Whether independent transformations is enabled.

    For PDFs sent through the pipeline, this enables independent transformations for
    each individual page. For CSVs, this enables transforming chunks of rows in the
    CSV.
    """

    output_schema: object = FieldInfo(alias="outputSchema")
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: str = FieldInfo(alias="outputSchemaName")
    """Name of output schema object."""

    next_action_type_config_id: Optional[str] = FieldInfo(alias="nextActionTypeConfigID", default=None)
    """Unique identifier of action type config to run after transformation.

    Currently only email is supported.
    """


class RouteConfig(ActionTypeConfigBase):
    action_type: Literal["route"] = FieldInfo(alias="actionType")

    routes: List[RouteListItem]
    """List of routes."""

    description: Optional[str] = None
    """Description of router.

    Can be used to provide additional context on router's purpose and expected
    inputs.
    """

    email_address: Optional[str] = FieldInfo(alias="emailAddress", default=None)
    """Email address automatically created by bem.

    You can forward emails with or without attachments, to be routed.
    """


class SplitConfigPrintPageSplitConfig(BaseModel):
    next_action_type_config_id: Optional[str] = FieldInfo(alias="nextActionTypeConfigID", default=None)
    """The unique ID of the action type configuration you want to use for this action."""


class SplitConfigSemanticPageSplitConfig(BaseModel):
    item_classes: Optional[List[SplitConfigSemanticPageItemClass]] = FieldInfo(alias="itemClasses", default=None)


class SplitConfig(ActionTypeConfigBase):
    action_type: Literal["split"] = FieldInfo(alias="actionType")

    split_type: Literal["print_page", "semantic_page"] = FieldInfo(alias="splitType")

    print_page_split_config: Optional[SplitConfigPrintPageSplitConfig] = FieldInfo(
        alias="printPageSplitConfig", default=None
    )

    semantic_page_split_config: Optional[SplitConfigSemanticPageSplitConfig] = FieldInfo(
        alias="semanticPageSplitConfig", default=None
    )


class JoinConfig(ActionTypeConfigBase):
    action_type: Literal["join"] = FieldInfo(alias="actionType")

    output_schema: object = FieldInfo(alias="outputSchema")
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: str = FieldInfo(alias="outputSchemaName")
    """Name of output schema object."""

    join_type: Optional[Literal["standard"]] = FieldInfo(alias="joinType", default=None)
    """The type of join to perform."""

    next_action_type_config_id: Optional[str] = FieldInfo(alias="nextActionTypeConfigID", default=None)
    """Unique identifier of action type config to run after join."""


class EmailConfig(ActionTypeConfigBase):
    action_type: Literal["email"] = FieldInfo(alias="actionType")

    body: str
    """Body of the email.

    This can be HTML, and include template variables in the form of
    `{{template_variable}}`. Template variables are taken from the output of the
    transformation.
    """

    from_email: str = FieldInfo(alias="fromEmail")
    """Email address to send the email from."""

    from_name: str = FieldInfo(alias="fromName")
    """Name of the sender."""

    subject: str
    """Subject of the email.

    This can include template variables in the form of `{{template_variable}}`.
    Template variables are taken from the output of the transformation.
    """

    to_email: str = FieldInfo(alias="toEmail")
    """Email address to send the email to."""

    to_name: str = FieldInfo(alias="toName")
    """Name of the recipient."""


ActionTypeConfig: TypeAlias = Annotated[
    Union[TransformConfig, RouteConfig, SplitConfig, JoinConfig, EmailConfig], PropertyInfo(discriminator="action_type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["SplitConfigSemanticPageItemClass"]


class SplitConfigSemanticPageItemClass(BaseModel):
    name: str

    description: Optional[str] = None

    next_action_type_config_id: Optional[str] = FieldInfo(alias="nextActionTypeConfigID", default=None)
    """The unique ID of the action type configuration you want to use for this action."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TaskStatus"]

TaskStatus: TypeAlias = Literal["running", "completed", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import List, Union, Mapping, Iterable, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import (
    transformation_list_params,
    transformation_create_params,
    transformation_delete_params,
    transformation_update_params,
    transformation_list_errors_params,
    transformation_deprecated_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.transformation_list_response import TransformationListResponse
from ..types.transformation_create_response import TransformationCreateResponse
from ..types.transformation_delete_response import TransformationDeleteResponse
from ..types.update_transformation_response import UpdateTransformationResponse
from ..types.transformation_list_errors_response import TransformationListErrorsResponse

__all__ = ["TransformationsResource", "AsyncTransformationsResource"]


class TransformationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransformationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransformationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransformationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return TransformationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        pipeline_id: str,
        transformations: Iterable[transformation_create_params.Transformation],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationCreateResponse:
        """
        Creates a batch of new transformations, each with a content and input type, and
        queues them up in bem's servers. Bem supports both `application/json` and
        `multipart/form-data` requests, where the latter is useful for large files.

        Args:
          pipeline_id: ID of the desired pipeline to transform the input content. Generated by bem.

          transformations: An array of objects containing all the transformations you want to create.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "pipeline_id": pipeline_id,
                "transformations": transformations,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v1-beta/transformations",
            body=maybe_transform(body, transformation_create_params.TransformationCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransformationCreateResponse,
        )

    def update(
        self,
        *,
        transformations: Iterable[transformation_update_params.Transformation] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UpdateTransformationResponse:
        """
        Updates a transformation with feedback on the corrected transformation value.

        Args:
          transformations: An array of objects containing all the transformations you want to patch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/v1-beta/transformations",
            body=maybe_transform(
                {"transformations": transformations}, transformation_update_params.TransformationUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UpdateTransformationResponse,
        )

    def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        item_offset: int | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        published: bool | NotGiven = NOT_GIVEN,
        published_after: Union[str, datetime] | NotGiven = NOT_GIVEN,
        published_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        transformation_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationListResponse:
        """
        Lists all performed transformations, with either pagination or specific
        reference IDs.

        Args:
          ending_before: A cursor to use in pagination. `endingBefore` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, starting with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call
              can include `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous
              page of the list.

          item_offset: Filters based on the index of the page of the document you had transformed. For
              CSV transformations, this offset represents starting row of the CSV for which
              you want to fetch transformations for. For PDF transformations, this offset
              represents the starting page of the PDF. This applies for a specific reference
              ID of a transformation. Note that this only applies if you specify one reference
              ID in `referenceIDs` and if you have `independentDocumentProcessingEnabled` set
              to `true` in your pipeline configuration. Note that this is zero-indexed.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          published: Boolean that toggles filtering whether or not transformations were successfully
              published via webhook.

          published_after: Filters to transformations successfully published via webhook after the
              specified date. Must be in RFC 3339 format.

          published_before: Filters to transformations successfully published via webhook before the
              specified date. Must be in RFC 3339 format.

          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can
              include `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of
              the list.

          transformation_ids: Array with the transform IDs of the transformed data points. Formatted as a CSV
              array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-beta/transformations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "item_offset": item_offset,
                        "limit": limit,
                        "pipeline_id": pipeline_id,
                        "published": published,
                        "published_after": published_after,
                        "published_before": published_before,
                        "reference_ids": reference_ids,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "transformation_ids": transformation_ids,
                    },
                    transformation_list_params.TransformationListParams,
                ),
            ),
            cast_to=TransformationListResponse,
        )

    def delete(
        self,
        *,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        transformation_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationDeleteResponse:
        """
        Deletes transformations by specifying pipeline ID, list of reference IDs, or
        list of transformation IDs. Will delete intersection of all params specified.

        Args:
          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          transformation_ids: Array with the transform IDs of the transformed data points. Formatted as a CSV
              array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/v1-beta/transformations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "pipeline_id": pipeline_id,
                        "reference_ids": reference_ids,
                        "transformation_ids": transformation_ids,
                    },
                    transformation_delete_params.TransformationDeleteParams,
                ),
            ),
            cast_to=TransformationDeleteResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def deprecated_update(
        self,
        *,
        transformations: Iterable[transformation_deprecated_update_params.Transformation] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UpdateTransformationResponse:
        """Deprecating will be replaced by put.

        Updates a transformation with feedback on
        the corrected transformation value.

        Args:
          transformations: An array of objects containing all the transformations you want to patch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/v1-beta/transformations",
            body=maybe_transform(
                {"transformations": transformations},
                transformation_deprecated_update_params.TransformationDeprecatedUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UpdateTransformationResponse,
        )

    def list_errors(
        self,
        *,
        reference_ids: List[str],
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationListErrorsResponse:
        """
        Lists all errors encountered while attempting to transform data, with either
        pagination or specific reference IDs.

        Args:
          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          ending_before: A cursor to use in pagination. `endingBefore` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, starting with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call
              can include `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can
              include `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of
              the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-beta/transformations/errors",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "reference_ids": reference_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "pipeline_id": pipeline_id,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    transformation_list_errors_params.TransformationListErrorsParams,
                ),
            ),
            cast_to=TransformationListErrorsResponse,
        )


class AsyncTransformationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransformationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransformationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransformationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncTransformationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        pipeline_id: str,
        transformations: Iterable[transformation_create_params.Transformation],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationCreateResponse:
        """
        Creates a batch of new transformations, each with a content and input type, and
        queues them up in bem's servers. Bem supports both `application/json` and
        `multipart/form-data` requests, where the latter is useful for large files.

        Args:
          pipeline_id: ID of the desired pipeline to transform the input content. Generated by bem.

          transformations: An array of objects containing all the transformations you want to create.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "pipeline_id": pipeline_id,
                "transformations": transformations,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v1-beta/transformations",
            body=await async_maybe_transform(body, transformation_create_params.TransformationCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransformationCreateResponse,
        )

    async def update(
        self,
        *,
        transformations: Iterable[transformation_update_params.Transformation] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UpdateTransformationResponse:
        """
        Updates a transformation with feedback on the corrected transformation value.

        Args:
          transformations: An array of objects containing all the transformations you want to patch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/v1-beta/transformations",
            body=await async_maybe_transform(
                {"transformations": transformations}, transformation_update_params.TransformationUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UpdateTransformationResponse,
        )

    async def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        item_offset: int | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        published: bool | NotGiven = NOT_GIVEN,
        published_after: Union[str, datetime] | NotGiven = NOT_GIVEN,
        published_before: Union[str, datetime] | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        transformation_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationListResponse:
        """
        Lists all performed transformations, with either pagination or specific
        reference IDs.

        Args:
          ending_before: A cursor to use in pagination. `endingBefore` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, starting with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call
              can include `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous
              page of the list.

          item_offset: Filters based on the index of the page of the document you had transformed. For
              CSV transformations, this offset represents starting row of the CSV for which
              you want to fetch transformations for. For PDF transformations, this offset
              represents the starting page of the PDF. This applies for a specific reference
              ID of a transformation. Note that this only applies if you specify one reference
              ID in `referenceIDs` and if you have `independentDocumentProcessingEnabled` set
              to `true` in your pipeline configuration. Note that this is zero-indexed.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          published: Boolean that toggles filtering whether or not transformations were successfully
              published via webhook.

          published_after: Filters to transformations successfully published via webhook after the
              specified date. Must be in RFC 3339 format.

          published_before: Filters to transformations successfully published via webhook before the
              specified date. Must be in RFC 3339 format.

          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can
              include `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of
              the list.

          transformation_ids: Array with the transform IDs of the transformed data points. Formatted as a CSV
              array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-beta/transformations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ending_before": ending_before,
                        "item_offset": item_offset,
                        "limit": limit,
                        "pipeline_id": pipeline_id,
                        "published": published,
                        "published_after": published_after,
                        "published_before": published_before,
                        "reference_ids": reference_ids,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "transformation_ids": transformation_ids,
                    },
                    transformation_list_params.TransformationListParams,
                ),
            ),
            cast_to=TransformationListResponse,
        )

    async def delete(
        self,
        *,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        transformation_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationDeleteResponse:
        """
        Deletes transformations by specifying pipeline ID, list of reference IDs, or
        list of transformation IDs. Will delete intersection of all params specified.

        Args:
          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          transformation_ids: Array with the transform IDs of the transformed data points. Formatted as a CSV
              array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/v1-beta/transformations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "pipeline_id": pipeline_id,
                        "reference_ids": reference_ids,
                        "transformation_ids": transformation_ids,
                    },
                    transformation_delete_params.TransformationDeleteParams,
                ),
            ),
            cast_to=TransformationDeleteResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def deprecated_update(
        self,
        *,
        transformations: Iterable[transformation_deprecated_update_params.Transformation] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UpdateTransformationResponse:
        """Deprecating will be replaced by put.

        Updates a transformation with feedback on
        the corrected transformation value.

        Args:
          transformations: An array of objects containing all the transformations you want to patch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/v1-beta/transformations",
            body=await async_maybe_transform(
                {"transformations": transformations},
                transformation_deprecated_update_params.TransformationDeprecatedUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UpdateTransformationResponse,
        )

    async def list_errors(
        self,
        *,
        reference_ids: List[str],
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        pipeline_id: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformationListErrorsResponse:
        """
        Lists all errors encountered while attempting to transform data, with either
        pagination or specific reference IDs.

        Args:
          reference_ids: Array with the reference IDs of the transformed data points. Formatted as a CSV
              array.

          ending_before: A cursor to use in pagination. `endingBefore` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, starting with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call
              can include `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          pipeline_id: The unique ID for a given pipeline. Will filter to just the transformations
              processed by the given pipeline. If left out, will query over ALL
              transformations for your account.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a transform ID that defines
              your place in the list. For example, if you make a list request and receive 50
              objects, ending with `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can
              include `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of
              the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-beta/transformations/errors",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "reference_ids": reference_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "pipeline_id": pipeline_id,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    transformation_list_errors_params.TransformationListErrorsParams,
                ),
            ),
            cast_to=TransformationListErrorsResponse,
        )


class TransformationsResourceWithRawResponse:
    def __init__(self, transformations: TransformationsResource) -> None:
        self._transformations = transformations

        self.create = to_raw_response_wrapper(
            transformations.create,
        )
        self.update = to_raw_response_wrapper(
            transformations.update,
        )
        self.list = to_raw_response_wrapper(
            transformations.list,
        )
        self.delete = to_raw_response_wrapper(
            transformations.delete,
        )
        self.deprecated_update = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                transformations.deprecated_update  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_errors = to_raw_response_wrapper(
            transformations.list_errors,
        )


class AsyncTransformationsResourceWithRawResponse:
    def __init__(self, transformations: AsyncTransformationsResource) -> None:
        self._transformations = transformations

        self.create = async_to_raw_response_wrapper(
            transformations.create,
        )
        self.update = async_to_raw_response_wrapper(
            transformations.update,
        )
        self.list = async_to_raw_response_wrapper(
            transformations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            transformations.delete,
        )
        self.deprecated_update = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                transformations.deprecated_update  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_errors = async_to_raw_response_wrapper(
            transformations.list_errors,
        )


class TransformationsResourceWithStreamingResponse:
    def __init__(self, transformations: TransformationsResource) -> None:
        self._transformations = transformations

        self.create = to_streamed_response_wrapper(
            transformations.create,
        )
        self.update = to_streamed_response_wrapper(
            transformations.update,
        )
        self.list = to_streamed_response_wrapper(
            transformations.list,
        )
        self.delete = to_streamed_response_wrapper(
            transformations.delete,
        )
        self.deprecated_update = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                transformations.deprecated_update  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_errors = to_streamed_response_wrapper(
            transformations.list_errors,
        )


class AsyncTransformationsResourceWithStreamingResponse:
    def __init__(self, transformations: AsyncTransformationsResource) -> None:
        self._transformations = transformations

        self.create = async_to_streamed_response_wrapper(
            transformations.create,
        )
        self.update = async_to_streamed_response_wrapper(
            transformations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            transformations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            transformations.delete,
        )
        self.deprecated_update = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                transformations.deprecated_update  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_errors = async_to_streamed_response_wrapper(
            transformations.list_errors,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config
from .input_type import InputType

__all__ = ["TransformationCreateParams", "Transformation"]


class TransformationCreateParams(TypedDict, total=False):
    pipeline_id: Required[Annotated[str, PropertyInfo(alias="pipelineID")]]
    """ID of the desired pipeline to transform the input content. Generated by bem."""

    transformations: Required[Iterable[Transformation]]
    """An array of objects containing all the transformations you want to create."""


class Transformation(TypedDict, total=False):
    input_content: Required[Annotated[Union[str, Base64FileInput], PropertyInfo(alias="inputContent", format="base64")]]
    """The file content you want to transform as a base64 URL-encoded string.

    If the `inputType` is `email`, this is equivalent to the raw format returned by
    the Gmail API.
    """

    input_type: Required[Annotated[InputType, PropertyInfo(alias="inputType")]]
    """The input type of the content you're sending for transformation."""

    reference_id: Required[Annotated[str, PropertyInfo(alias="referenceID")]]
    """The unique ID you use internally to refer to this data point.

    Upon successful transformation, we will return it to you with the transformed
    content, so you can associate inputs and outputs.
    """


set_pydantic_config(Transformation, {"arbitrary_types_allowed": True})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ActionBase"]


class ActionBase(BaseModel):
    action_id: str = FieldInfo(alias="actionID")

    action_type_config_id: str = FieldInfo(alias="actionTypeConfigID")

    reference_id: str = FieldInfo(alias="referenceID")

    status: Literal["pending", "running", "completed", "failed"]
    """The status of the action."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .action_type import ActionType
from .join_action_base_param import JoinActionBaseParam
from .email_action_base_param import EmailActionBaseParam
from .route_action_base_param import RouteActionBaseParam
from .split_action_base_param import SplitActionBaseParam
from .create_action_base_param import CreateActionBaseParam
from .transform_action_base_param import TransformActionBaseParam

__all__ = [
    "ActionCreateParams",
    "CreateTransformActions",
    "CreateTransformActionsAction",
    "CreateRouteActions",
    "CreateRouteActionsAction",
    "CreateSplitActions",
    "CreateSplitActionsAction",
    "CreateJoinActions",
    "CreateJoinActionsAction",
    "CreateEmailActions",
    "CreateEmailActionsAction",
]


class CreateTransformActions(TypedDict, total=False):
    actions: Required[Iterable[CreateTransformActionsAction]]

    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]


class CreateTransformActionsAction(CreateActionBaseParam, TransformActionBaseParam, total=False):
    pass


class CreateRouteActions(TypedDict, total=False):
    actions: Required[Iterable[CreateRouteActionsAction]]

    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]


class CreateRouteActionsAction(CreateActionBaseParam, RouteActionBaseParam, total=False):
    pass


class CreateSplitActions(TypedDict, total=False):
    actions: Required[Iterable[CreateSplitActionsAction]]

    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]


class CreateSplitActionsAction(CreateActionBaseParam, SplitActionBaseParam, total=False):
    pass


class CreateJoinActions(TypedDict, total=False):
    actions: Required[Iterable[CreateJoinActionsAction]]

    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]


class CreateJoinActionsAction(CreateActionBaseParam, JoinActionBaseParam, total=False):
    pass


class CreateEmailActions(TypedDict, total=False):
    action_type: Required[Annotated[ActionType, PropertyInfo(alias="actionType")]]
    """The type of the action."""

    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]

    actions: Iterable[CreateEmailActionsAction]


class CreateEmailActionsAction(CreateActionBaseParam, EmailActionBaseParam, total=False):
    pass


ActionCreateParams: TypeAlias = Union[
    CreateTransformActions, CreateRouteActions, CreateSplitActions, CreateJoinActions, CreateEmailActions
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, cast
from typing_extensions import Literal

import httpx

from ..types import event_list_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.event import Event
from .._base_client import make_request_options
from ..types.event_list_response import EventListResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Event:
        """
        Get an Event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return cast(
            Event,
            self._get(
                f"/v1-alpha/events/{event_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Event),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        event_ids: List[str] | NotGiven = NOT_GIVEN,
        event_types: List[Literal["transform", "route", "split_collection", "split_item", "error", "join"]]
        | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventListResponse:
        """List Events

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is an event ID that defines your
              place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an event ID that defines your
              place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-alpha/events",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "event_ids": event_ids,
                        "event_types": event_types,
                        "limit": limit,
                        "reference_ids": reference_ids,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            cast_to=EventListResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Event:
        """
        Get an Event

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return cast(
            Event,
            await self._get(
                f"/v1-alpha/events/{event_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, Event),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        event_ids: List[str] | NotGiven = NOT_GIVEN,
        event_types: List[Literal["transform", "route", "split_collection", "split_item", "error", "join"]]
        | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        reference_ids: List[str] | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventListResponse:
        """List Events

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is an event ID that defines your
              place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an event ID that defines your
              place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-alpha/events",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "event_ids": event_ids,
                        "event_types": event_types,
                        "limit": limit,
                        "reference_ids": reference_ids,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            cast_to=EventListResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = to_raw_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            events.list,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            events.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransformationDeleteParams"]


class TransformationDeleteParams(TypedDict, total=False):
    pipeline_id: Annotated[str, PropertyInfo(alias="pipelineID")]
    """The unique ID for a given pipeline.

    Will filter to just the transformations processed by the given pipeline. If left
    out, will query over ALL transformations for your account.
    """

    reference_ids: Annotated[List[str], PropertyInfo(alias="referenceIDs")]
    """Array with the reference IDs of the transformed data points.

    Formatted as a CSV array.
    """

    transformation_ids: Annotated[List[str], PropertyInfo(alias="transformationIDs")]
    """Array with the transform IDs of the transformed data points.

    Formatted as a CSV array.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["EventBase"]


class EventBase(BaseModel):
    event_id: str = FieldInfo(alias="eventID")
    """Unique ID generated by bem to identify the event."""

    reference_id: str = FieldInfo(alias="referenceID")
    """
    The unique ID you use internally to refer to this data point, propagated from
    the original action input.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransformationDeprecatedUpdateParams", "Transformation"]


class TransformationDeprecatedUpdateParams(TypedDict, total=False):
    transformations: Iterable[Transformation]
    """An array of objects containing all the transformations you want to patch."""


class Transformation(TypedDict, total=False):
    corrected_json: Annotated[Optional[object], PropertyInfo(alias="correctedJSON")]
    """The object with properties of the transformation that you want updated."""

    order_matching: Annotated[Optional[bool], PropertyInfo(alias="orderMatching")]
    """True if order in the array matters. Default is false."""

    transformation_id: Annotated[str, PropertyInfo(alias="transformationID")]
    """The unique ID you use internally to refer to a transform."""
